TRANSLITERATION_APP_URL = "api/transliteration/v1/transliterate"
TRANSLITERATION_LANGUAGE_CATALOG_URL = "api/transliteration/v1/supported-languages"
TRANSLIT_TEXT_COMMAND = "neuralspace transliteration parse --help"
